﻿Public Class Mob2J
    'Structure declarations
    Structure JprStructure      'Necessary JPR data (in the context of this application)
        Public nm As String         'Name of chart
        Public sc As Double         'Scale
        Public sr As Double         'Resolution
        Public ds As Double         'Size of a pixel in meter
        Public it As String         'Type op image
        Public dm As String         'Datum
        Public st As Double         'Datum shift (latitude)
        Public sn As Double         'Datum shift (longitude)
        Public pr As String         'Projection
        Public pp As String         'Central meridian
        Public p2 As String         'Scale Factor
        Public p3 As String         'False Northing
        Public p4 As String         'False Easting
    End Structure

    Structure Coordinate        'Referenced Pixel and Geographical coordinates
        Public x As String          'Pixel X
        Public y As String          'Pixel Y
        Public lat As String        'Latitude
        Public lon As String        'Longitude
    End Structure

    'Variable declarations
    Public MapFilePath As String                    'Filename and path to MAP-file
    Public MapFile As String                        'Filename MAP-file
    Public MapPath As String                        'Path to MAP-file
    Public JprFilePath As String                    'Filename and path to JPR-file.
    Public ChartName As String                      'MAP-file name without extension (ChartName was used to prevent confusion with the use of map-file)
    Public MapFileContent As String                 'Content of .map file
    Public LineCounter As Integer                   'Amount of lines in MapFileContent (will be (re)calculated for every map)
    Public LineContent(60) As String                'Content of each line in MapFileContent (will be (re)dimensioned for every map)
    Public LineField(4) As String                   'Content of a field in a line (used to extract coordinates)
    Public JprData As JprStructure                  'Variable representing the JPR data
    Public ReferencePoint(4) As Coordinate                'Reference points for calibration and vertex polygon (topleft, topright, bottomright, bottomleft)

    Public EndLine As String = Chr(13) + Chr(10)    'Variable representing the end a line of text.

    'subroutine Mob2J_Load will be executed starting the application
    '- It enables and sets the Tooltips for buttons and boxes.
    '- Loading DpiBox with default values
    '- It enables/disables buttons
    Private Sub Mob2J_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        'Declarations
        Dim TipText As New ToolTip()

        'Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        'Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        ' Setting up the TipText text 
        TipText.SetToolTip(Me.FindButton, "Find and select a MAP-file.")
        TipText.SetToolTip(Me.MapInfoBox, "Content of the selected MAP-file.")
        TipText.SetToolTip(Me.BuildButton, "Build the content for the JPR-file.")
        TipText.SetToolTip(Me.DpiBox, "Select or change the resolution of the chart, 254 dpi (100 dot per cm) is the default.")
        TipText.SetToolTip(Me.JprFileBox, "Content for the JPR-file.")
        TipText.SetToolTip(Me.SaveButton, "Save the content for JPR-file in the same directory as the MAP-file. If the JPR-file exists, you wil be prompted to over write.")
        TipText.SetToolTip(Me.DisclaimButton, "Disclaimer for the application Mobac MAP data to JPR (M2J).")
        TipText.SetToolTip(Me.ClearButton, "Clear all data with in the applicatation, except the selected resolution.")
        TipText.SetToolTip(Me.EndButton, "Close the application.")

        'Loading DpiBox
        DpiBox.Items.Add("72")
        DpiBox.Items.Add("96")
        DpiBox.Items.Add("100")
        DpiBox.Items.Add("127")
        DpiBox.Items.Add("150")
        DpiBox.Items.Add("200")
        DpiBox.Items.Add("254")
        DpiBox.Items.Add("300")
        DpiBox.Items.Add("508")
        'Default value for DpiBox
        DpiBox.Text = 254

        'Initial setting user interface (buttons)
        BuildButton.Enabled = False
        SaveButton.Enabled = False
        DpiBox.Enabled = False

    End Sub

    'Subroutine FindButton_Click
    '- Finding/Selecting and opening a MAP file
    '- Creating values for variables: MapFile, MapPath, JprFileName, MapFileContent
    '- Presenting MapFileContent on the screen
    '- Loading MapFileContent of MapFile into an array (LineContent)
    Private Sub FindButton_Click(sender As Object, e As EventArgs) Handles FindButton.Click
        'Declarations
        Dim FileNameLen, Pointer As Integer
        Dim TempFileName As String
        Dim MapFileContentLen, ChrPointer, LinePointer As Integer
        Dim TxtChr As String = ""
        Dim FindFile As New OpenFileDialog

        'Clearing all data on screen and variables
        Call ClearButton_Click(sender, e)

        'Setting up dialog 
        FindFile.Filter = "Ozi-MAP file (*.map) | *.map"
        FindFile.Multiselect = False
        'Opening dialog
        FindFile.ShowDialog()

        'Retrieving the value for MAPfile
        TempFileName = FindFile.FileName

        'Stopping this subroutine if no file was selected (preventing an error)
        '(length TempfileNam = 0)
        If Len(TempFileName) = 0 Then
            MsgBox("No file was selected!", 48, "Warning")
            Exit Sub
        End If

        'Setting values for variables
        MapFilePath = TempFileName
        MapFileValue.Text = TempFileName

        'Retrieving the value for MapFile and MapPath
        FileNameLen = Len(TempFileName)
        For Pointer = FileNameLen To 0 Step -1
            If Mid(TempFileName, Pointer, 1) = "\" Then
                MapPath = Mid(TempFileName, 1, Pointer)
                MapFile = Mid(TempFileName, Pointer + 1, FileNameLen - Pointer)
                Exit For
            End If
        Next Pointer

        'Retrieving the value for ChartName and JprFileName
        FileNameLen = Len(MapFile)
        For Pointer = FileNameLen To 0 Step -1
            If Mid(MapFile, Pointer, 1) = "." Then
                ChartName = Mid(MapFile, 1, Pointer - 1)
                JprFilePath = MapPath & ChartName & ".jpr"
                Exit For
            End If
        Next Pointer

        'Retreiving content of the MapFile
        MapFileContent = My.Computer.FileSystem.ReadAllText(MapPath + MapFile)

        'Loading MapFileContent into MapInfoBox
        MapInfoBox.Text = MapFileContent

        'Breaking MapFileContent into Lines (in array LineContent)
        '- Determening amount of lines (LineCounter)
        MapFileContentLen = Len(MapFileContent)
        LineCounter = 1
        For I = 1 To MapFileContentLen
            If Mid(MapFileContent, I, 1) = Chr(13) Then
                LineCounter = LineCounter + 1
            End If
        Next
        '- Retrievint the lines and put them in the array LineContent
        ReDim LineContent(LineCounter)
        ChrPointer = 1
        LinePointer = 1
        Do While LinePointer < LineCounter And ChrPointer < MapFileContentLen
            Do
                TxtChr = Mid(MapFileContent, ChrPointer, 1)
                If TxtChr <> Chr(13) And TxtChr <> Chr(10) Then
                    LineContent(LinePointer) = LineContent(LinePointer) + TxtChr
                End If
                ChrPointer = ChrPointer + 1
            Loop Until TxtChr = Chr(13) Or TxtChr = Chr(10)
            If TxtChr = Chr(13) Then LinePointer = LinePointer + 1
        Loop

        'Enabling BuildButton (to build the content of JPR file)
        BuildButton.Enabled = True
        BuildButton.Select()

    End Sub

    'Subroutine BuildButton_Click
    '- Retreiving all data for JPR File
    '- Writing all data to JprFileBox in the correct structure
    Private Sub BuildButton_Click(sender As Object, e As EventArgs) Handles BuildButton.Click
        'Retrieving calibrationpoints (ReferencePoint) and Size of a pixel in meter (JprData.ds)
        For LinePointer = 41 To LineCounter
            Call SplitLine(LineContent(LinePointer))
            Select Case LCase(LineField(1))
                Case "mmpll"        'latitude and and longitude
                    ReferencePoint(Val(LineField(2))).lon = Trim(LineField(3))
                    ReferencePoint(Val(LineField(2))).lat = Trim(LineField(4))
                Case "mmpxy"        'pixel X and Y 
                    ReferencePoint(Val(LineField(2))).x = Trim(LineField(3))
                    ReferencePoint(Val(LineField(2))).y = Trim(LineField(4))
                Case "mm1b"         'Size of pixel in real in meter
                    JprData.ds = Val(LineField(2))
            End Select
        Next

        'Retrieving the data for variable JprData
        JprData.nm = LCase(Trim(LineContent(2)))
        JprData.dm = "wgs84"
        JprData.st = 0
        JprData.sn = 0
        JprData.pr = "mercator"
        JprData.pp = "0"
        JprData.p2 = "1"
        JprData.p3 = "0"
        JprData.p4 = "0"
        JprData.it = "png"
        JprData.sr = Val(DpiBox.Text)
        JprData.sc = Int((JprData.sr / 2.54) * 100 * JprData.ds)

        'Writing the data to JprFileBox
        JprFileBox.Text = "//This file was created using Mobac MAP data to JPR (Mob2J) by www.hzns.nl" & EndLine &
            "nm=" & JprData.nm & EndLine &
            "dm=" & JprData.dm & EndLine &
            "st=" & JprData.st & EndLine &
            "sn=" & JprData.sn & EndLine &
            "pr=" & JprData.pr & EndLine &
            "pp=" & JprData.pp & EndLine &
            "p2=" & JprData.p2 & EndLine &
            "p3=" & JprData.p3 & EndLine &
            "p4=" & JprData.p4 & EndLine &
            "it=" & JprData.it & EndLine &
            "sr=" & JprData.sr & EndLine &
            "sc=" & JprData.sc & EndLine &
            "rp1=" & ReferencePoint(1).lat & "," & ReferencePoint(1).lon & "," & ReferencePoint(1).x & "," & ReferencePoint(1).y & EndLine &
            "rp2=" & ReferencePoint(2).lat & "," & ReferencePoint(2).lon & "," & ReferencePoint(2).x & "," & ReferencePoint(2).y & EndLine &
            "rp3=" & ReferencePoint(3).lat & "," & ReferencePoint(3).lon & "," & ReferencePoint(3).x & "," & ReferencePoint(3).y & EndLine &
            "rp4=" & ReferencePoint(4).lat & "," & ReferencePoint(4).lon & "," & ReferencePoint(4).x & "," & ReferencePoint(4).y & EndLine &
            "vp1=" & ReferencePoint(1).x & "," & ReferencePoint(1).y & EndLine &
            "vp2=" & ReferencePoint(2).x & "," & ReferencePoint(2).y & EndLine &
            "vp3=" & ReferencePoint(3).x & "," & ReferencePoint(3).y & EndLine &
            "vp4=" & ReferencePoint(4).x & "," & ReferencePoint(4).y & EndLine

        'Enabling SaveButton (to save the content of JPR file)
        DpiBox.Enabled = True
        SaveButton.Enabled = True
        SaveButton.Select()
    End Sub

    'Subroutine SaveButton_Click 
    '-  Saving the JrpFileContentBox.text to a JPR-file in the same directory 
    '   as the orginal MAP file. If this JPR file exists a warning will be prompted.
    Private Sub SaveButton_Click(sender As Object, e As EventArgs) Handles SaveButton.Click
        'Declarations
        Dim Answer As Integer       'Used as answer value for MsgBox

        'Checking for an existing file
        If My.Computer.FileSystem.FileExists(JprFilePath) = True Then
            Answer = MsgBox("JPR-file allready exits. Overwrite?", 292, "Warning")
            If Answer = 6 Then
                'Overwriting existing JPR-file if answer on MsgBox = Yes (Answer = 6)
                My.Computer.FileSystem.WriteAllText(JprFilePath, JprFileBox.Text, False)
            End If
        Else
            'Creating JPR-file
            My.Computer.FileSystem.WriteAllText(JprFilePath, JprFileBox.Text, False)
        End If

    End Sub

    'Subroutine DisclaimerButton_Click
    '- Showing the disclaimer
    Private Sub DisclaimerButton_Click(sender As Object, e As EventArgs) Handles DisclaimButton.Click
        'Declarations
        Dim MsgText As String

        'Disclaimer message
        MsgText = "This application Is provided 'As is'. The use of the application is on your own risk. " &
            "Direct or indirect damage by using this application is users responsibility, " &
            "not the application-builders. Redistribution by a third party (commercial of non-commercial) " &
            "is prohibit. Please download the application direct from www.hzns.nl."
        MsgBox(MsgText, 48, "Disclaimer")

    End Sub

    'Subroutine ClearButton_Click
    '- Clearing all data from screen
    '- disabling BuildButton and SaveButton
    Private Sub ClearButton_Click(sender As Object, e As EventArgs) Handles ClearButton.Click
        'Declarations
        Dim Pointer As Integer

        'Clearing textboxes and labels
        MapInfoBox.Text = ""
        JprFileBox.Text = ""
        MapFileValue.Text = ""

        'Disabeling buttons
        SaveButton.Enabled = False
        BuildButton.Enabled = False
        DpiBox.Enabled = False

        'Clearing MAP file Data
        MapFile = ""
        MapPath = ""
        MapFilePath = ""
        ChartName = ""
        JprFilePath = ""
        MapFileContent = ""
        For Pointer = 0 To LineCounter
            LineContent(Pointer) = ""
        Next

    End Sub

    'Suboutine EndButton_Click
    '- Closing application
    Private Sub EndButton_Click(sender As Object, e As EventArgs) Handles EndButton.Click
        'Unloading and closing application
        Me.Close()

    End Sub

    'Subroutine SplitLine
    '- Splitting a line With comma separeted fields into 4 fields
    '  (used for calibration data)
    Private Sub SplitLine(LineText As String)
        'Declarations
        Dim FieldPointer, ChrPointer As Integer
        Dim TxtChr As String

        'Clearing all information Array LineField
        For FieldPointer = 0 To 4
            LineField(FieldPointer) = ""
        Next
        'Splitting line into fields
        FieldPointer = 1
        ChrPointer = 1
        Do While ChrPointer <= Len(LineText)
            Do
                TxtChr = Mid(LineText, ChrPointer, 1)
                If TxtChr <> Chr(44) Then
                    LineField(FieldPointer) = LineField(FieldPointer) + TxtChr
                End If
                ChrPointer = ChrPointer + 1
            Loop Until TxtChr = Chr(44) Or ChrPointer > Len(LineText)
            If TxtChr = Chr(44) Then FieldPointer = FieldPointer + 1
        Loop
    End Sub

    'Subroutine DipBox_LostFocus handles the cases LostFocus and SelectedValueChanged (with focus on DpiBox)
    '- recalculating the map scale
    Private Sub DpiBox_LostFocus(sender As Object, e As EventArgs) Handles DpiBox.LostFocus, DpiBox.SelectedValueChanged
        '(only if JprFileBox contains information)
        If JprFileBox.Text <> "" Then
            If JprData.sr <> Val(DpiBox.Text) Then
                Call BuildButton_Click(sender, e) '(learned by trail and error)
                SaveButton.Select()
            End If
        End If

    End Sub

    'Subroutine DipBox_PressKey handles the case "Enter"-key was used (with focus on DpiBox)
    '- recalculaing the map scale
    Private Sub DpiBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DpiBox.KeyPress
        If JprFileBox.Text <> "" And e.KeyChar = Chr(13) Then
            If JprData.sr <> Val(DpiBox.Text) Then
                JprData.sr = Val(DpiBox.Text)
                Call BuildButton_Click(sender, e) '(learned by trail and error)
            End If
        End If

    End Sub

End Class
